Option Compare Binary
Option Explicit On
Option Strict On

Imports System.Runtime.InteropServices

Public NotInheritable Class DllVersionInfo
    Private Delegate Function DllGetVersion(<Out()> ByRef pdvi As DLLVERSIONINFO_) As Int32

    <StructLayout(LayoutKind.Sequential)> _
    Private Structure DLLVERSIONINFO_
        Public cbSize As Int32
        Public dwMajorVersion As Int32
        Public dwMinorVersion As Int32
        Public dwBuildNumber As Int32
        Public dwPlatformID As DLLVER_PLATFORM
    End Structure

    Private Enum DLLVER_PLATFORM
        DLLVER_PLATFORM_WINDOWS = &H1
        DLLVER_PLATFORM_NT = &H2
    End Enum

    Private Const NOERROR As Int32 = 0

    Public Enum PlatformID
        Windows = DLLVER_PLATFORM.DLLVER_PLATFORM_WINDOWS
        NT = DLLVER_PLATFORM.DLLVER_PLATFORM_NT
    End Enum

    Private m_d As DLLVERSIONINFO_

    Private Sub New(ByVal FileName As String)
        m_d = GetDllVersionInfo(FileName)
    End Sub

    Public Shared Function GetVersionInfo(ByVal FileName As String) As DllVersionInfo
        Return New DllVersionInfo(FileName)
    End Function

    Public ReadOnly Property Platform() As PlatformID
        Get
            Return CType(m_d.dwPlatformID, PlatformID)
        End Get
    End Property

    Public ReadOnly Property BuildPart() As Integer
        Get
            Return m_d.dwBuildNumber
        End Get
    End Property

    Public ReadOnly Property MinorPart() As Integer
        Get
            Return m_d.dwMinorVersion
        End Get
    End Property

    Public ReadOnly Property MajorPart() As Integer
        Get
            Return m_d.dwMajorVersion
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return m_d.dwMajorVersion & "." & m_d.dwMinorVersion & "." & m_d.dwBuildNumber & " (" & m_d.dwPlatformID.ToString() & ")"
    End Function

    Private Function GetDllVersionInfo( _
        ByVal FileName As String _
    ) As DLLVERSIONINFO_
        Using Library As NativeLibrary = NativeLibrary.LoadFrom(FileName)
            Dim d As DLLVERSIONINFO_
            d.cbSize = Marshal.SizeOf(d)
            Dim Args() As Object = {d}
            Dim hr As Integer = _
                CInt( _
                    Library.Invoke( _
                        "DllGetVersion", _
                        GetType(DllGetVersion), _
                        Args _
                    ) _
                )
            If hr <> NOERROR Then
                Marshal.ThrowExceptionForHR(hr)
            End If
            d = CType(Args(0), DLLVERSIONINFO_)
            Return d
        End Using
    End Function
End Class