Option Compare Binary
Option Explicit On
Option Strict On

Imports System.ComponentModel
Imports System.Runtime.InteropServices

Public NotInheritable Class NativeLibrary
    Implements IDisposable

    Private Declare Auto Function LoadLibrary Lib "kernel32.dll" ( _
        ByVal lpLibFileName As String _
    ) As IntPtr

    Private Declare Function FreeLibrary Lib "kernel32.dll" ( _
        ByVal hLibModule As IntPtr _
    ) As Boolean

    ' Call by name.
    Private Declare Function GetProcAddress Lib "kernel32.dll" ( _
        ByVal hModule As IntPtr, _
        ByVal lpProcName As String _
    ) As IntPtr

    ' Call by ordinal.
    Private Declare Function GetProcAddress Lib "kernel32.dll" ( _
        ByVal hModule As IntPtr, _
        ByVal lpProcName As Int32 _
    ) As IntPtr

    'Private Declare Auto Function GetModuleHandleEx Lib "kernel32.dll" ( _
    '    ByVal dwFlags As GET_MODULE_HANDLE_EX_FLAG, _
    '    ByVal lpModuleName As String, _
    '    ByRef phModule As IntPtr _
    ') As Boolean

    '<Flags()> _
    'Private Enum GET_MODULE_HANDLE_EX_FLAG
    '    [Default] = &H0
    '    GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS = &H4
    '    GET_MODULE_HANDLE_EX_FLAG_PIN = &H1
    '    GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT = &H2
    'End Enum

    ' Requires Windows XP SP1 or Windows Server 2003.
    Private Declare Auto Function SetDllDirectory Lib "kernel32.dll" ( _
        ByVal lpPathName As String _
    ) As Boolean

    Private m_hModule As IntPtr

    Private Sub New(ByVal FileName As String)
        m_hModule = LoadLibrary(FileName)
        If m_hModule = IntPtr.Zero Then
            Throw New Win32Exception()
        End If
    End Sub

    Private Sub New(ByVal FileName As String, ByVal CustomSearchPath As String)

        ' Check if Windows XP SP1 or Windows Server 2003.
        If Environment.OSVersion.Version.CompareTo(New Version(5, 2)) < 0 Then
            Throw New NotSupportedException("Setting the custom search path is not supported by the system.")
        End If

        ' Add custom search path.
        If Not SetDllDirectory(CustomSearchPath) Then
            Throw New Win32Exception()
        End If

        m_hModule = LoadLibrary(FileName)

        ' Remove custom search path.
        If Not SetDllDirectory(Nothing) Then
            Throw New Win32Exception()
        End If

        If m_hModule = IntPtr.Zero Then
            Throw New Win32Exception()
        End If
    End Sub

    Public Shared Function LoadFrom( _
        ByVal FileName As String _
    ) As NativeLibrary
        Return New NativeLibrary(FileName)
    End Function

    Public Shared Function LoadFrom( _
        ByVal FileName As String, _
        ByVal CustomSearchPath As String _
    ) As NativeLibrary
        Return New NativeLibrary(FileName, CustomSearchPath)
    End Function

    Public Function Invoke( _
        ByVal ProcName As String, _
        ByVal DelegateType As Type, _
        ByVal Args() As Object _
    ) As Object
        Dim p As IntPtr = GetProcAddress(m_hModule, ProcName)
        If p = IntPtr.Zero Then
            Throw New Win32Exception()
        Else
            Dim d As [Delegate] = _
                Marshal.GetDelegateForFunctionPointer(p, DelegateType)
            Return d.DynamicInvoke(Args)
        End If
    End Function

    Public Function Invoke( _
        ByVal ProcOrdinal As Integer, _
        ByVal DelegateType As Type, _
        ByVal Args() As Object _
    ) As Object
        Dim p As IntPtr = GetProcAddress(m_hModule, ProcOrdinal)
        If p = IntPtr.Zero Then
            Throw New Win32Exception()
        Else
            Dim d As [Delegate] = _
                Marshal.GetDelegateForFunctionPointer(p, DelegateType)
            Return d.DynamicInvoke(Args)
        End If
    End Function

    Public Function GetProcAddress(ByVal ProcName As String) As IntPtr
        Dim p As IntPtr = GetProcAddress(m_hModule, ProcName)
        If p = IntPtr.Zero Then
            Throw New Win32Exception()
        Else
            Return p
        End If
    End Function

    Public Function GetProcAddress(ByVal ProcOrdinal As Integer) As IntPtr
        Dim p As IntPtr = GetProcAddress(m_hModule, ProcOrdinal)
        If p = IntPtr.Zero Then
            Throw New Win32Exception()
        Else
            Return p
        End If
    End Function

    Private Function IsExported( _
        ByVal ProcName As String _
    ) As Boolean
        Return GetProcAddress(m_hModule, ProcName) = IntPtr.Zero
    End Function

    Private Function IsExported( _
        ByVal ProcOrdinal As Integer _
    ) As Boolean
        Return GetProcAddress(m_hModule, ProcOrdinal) = IntPtr.Zero
    End Function

    Public Sub Free() Implements IDisposable.Dispose
        If Not FreeLibrary(m_hModule) Then
            Throw New Win32Exception()
        End If
    End Sub
End Class