Option Compare Binary
Option Explicit On
Option Strict On

Imports System.ComponentModel
Imports System.Runtime.InteropServices

Public Class ActivationContext
    Implements IDisposable

    Private m_hActCtx As IntPtr
    Private m_Cookies As New Stack(Of ActivationContextCookie)()

    Private Shared m_ActivationContexts As New List(Of ActivationContext)()

    Public Shared Function Create(ByVal ManifestFileName As String, ByVal AssemblyDirectory As String, Optional ByVal Activate As Boolean = True) As ActivationContext
        Return New ActivationContext(CreateActCtx(IntPtr.Zero, Nothing, AssemblyDirectory, Nothing, ManifestFileName, 0, 0), Activate)
    End Function

    Public Shared Function Create(ByVal FileName As String, ByVal AssemblyDirectory As String, ByVal ResourceName As String, Optional ByVal Activate As Boolean = True) As ActivationContext
        Return New ActivationContext(CreateActCtx(IntPtr.Zero, Nothing, AssemblyDirectory, ResourceName, FileName, 0, 0), Activate)
    End Function

    Public Shared Function Create(ByVal ModuleHandle As IntPtr, ByVal AssemblyDirectory As String, ByVal ResourceName As String, Optional ByVal Activate As Boolean = True) As ActivationContext
        Return New ActivationContext(CreateActCtx(ModuleHandle, Nothing, AssemblyDirectory, ResourceName, Nothing, 0, 0), Activate)
    End Function

    Public Shared Function Create(ByVal FileName As String, ByVal AssemblyDirectory As String, ByVal ResourceID As Short, Optional ByVal Activate As Boolean = True) As ActivationContext
        Return New ActivationContext(CreateActCtx(IntPtr.Zero, Nothing, AssemblyDirectory, NativeMethods.MAKEINTRESOURCE(ResourceID), Nothing, 0, 0), Activate)
    End Function

    Public Shared Function Create(ByVal ModuleHandle As IntPtr, ByVal AssemblyDirectory As String, ByVal ResourceId As Short, Optional ByVal Activate As Boolean = True) As ActivationContext
        Return New ActivationContext(CreateActCtx(ModuleHandle, Nothing, AssemblyDirectory, NativeMethods.MAKEINTRESOURCE(ResourceID), Nothing, 0, 0), Activate)
    End Function

    Public Shared Function Create(ByVal ModuleHandle As IntPtr, ByVal ApplicationName As String, ByVal AssemblyDirectory As String, ByVal ResourceName As String, ByVal FileName As String, ByVal LangId As Short, ByVal ProcessorArchitecture As ProcessorArchitecture, Optional ByVal Activate As Boolean = True) As ActivationContext
        Return New ActivationContext(CreateActCtx(ModuleHandle, ApplicationName, AssemblyDirectory, ResourceName, FileName, LangID, CType(ProcessorArchitecture, NativeMethods.PROCESSOR_ARCHITECTURE)), Activate)
    End Function

    Public Shared Function Create(ByVal ModuleHandle As IntPtr, ByVal ApplicationName As String, ByVal AssemblyDirectory As String, ByVal ResourceId As Short, ByVal FileName As String, ByVal LangId As Short, ByVal ProcessorArchitecture As ProcessorArchitecture, Optional ByVal Activate As Boolean = True) As ActivationContext
        Return New ActivationContext(CreateActCtx(ModuleHandle, ApplicationName, AssemblyDirectory, NativeMethods.MAKEINTRESOURCE(ResourceId), FileName, LangId, CType(ProcessorArchitecture, NativeMethods.PROCESSOR_ARCHITECTURE)), Activate)
    End Function

    Private Shared Function CreateActCtx(ByVal hModule As IntPtr, ByVal lpApplicationName As String, ByVal lpAssemblyDirectory As String, ByVal lpResourceName As String, ByVal lpSource As String, ByVal wLangId As Int16, ByVal wProcessorArchitecture As NativeMethods.PROCESSOR_ARCHITECTURE) As NativeMethods.ACTCTX
        Dim ActCtx As New NativeMethods.ACTCTX
        With ActCtx
            .dwFlags = NativeMethods.ACTCTX_FLAG.ACTCTX_FLAG_ASSEMBLY_DIRECTORY_VALID
            If hModule <> IntPtr.Zero Then
                .dwFlags = .dwFlags Or NativeMethods.ACTCTX_FLAG.ACTCTX_FLAG_HMODULE_VALID
            End If
            If lpApplicationName IsNot Nothing Then
                .dwFlags = .dwFlags Or NativeMethods.ACTCTX_FLAG.ACTCTX_FLAG_APPLICATION_NAME_VALID
            End If
            If lpAssemblyDirectory IsNot Nothing Then
                .dwFlags = .dwFlags Or NativeMethods.ACTCTX_FLAG.ACTCTX_FLAG_ASSEMBLY_DIRECTORY_VALID
            End If
            If lpResourceName IsNot Nothing Then
                .dwFlags = .dwFlags Or NativeMethods.ACTCTX_FLAG.ACTCTX_FLAG_RESOURCE_NAME_VALID
            End If
            If wLangId <> 0 Then
                .dwFlags = .dwFlags Or NativeMethods.ACTCTX_FLAG.ACTCTX_FLAG_LANGID_VALID
            End If
            If wProcessorArchitecture <> 0 Then
                .dwFlags = .dwFlags Or NativeMethods.ACTCTX_FLAG.ACTCTX_FLAG_PROCESSOR_ARCHITECTURE_VALID
            End If
            .hModule = hModule
            .lpApplicationName = lpApplicationName
            .lpAssemblyDirectory = lpAssemblyDirectory
            .lpResourceName = lpResourceName
            .lpSource = lpSource
            .wLangId = wLangId
            .wProcessorArchitecture = wProcessorArchitecture
            .cbSize = CUInt(Marshal.SizeOf(ActCtx))
        End With
        Return ActCtx
    End Function

    Private Sub New(ByRef ActCtx As NativeMethods.ACTCTX, ByVal Activate As Boolean)
        m_ActivationContexts.Add(Me)
        m_hActCtx = NativeMethods.CreateActCtx(ActCtx)
        If m_hActCtx = NativeMethods.INVALID_HANDLE_VALUE Then
            Throw New Win32Exception()
        End If
        If Activate Then
            Me.Activate()
        End If
    End Sub

    Public Sub Activate()
        Dim Cookie As UInt32
        If Not NativeMethods.ActivateActCtx(m_hActCtx, Cookie) Then
            Throw New Win32Exception()
        End If
        m_Cookies.Push(New ActivationContextCookie(Cookie))
    End Sub

    Public Sub Deactivate()
        If m_Cookies.Count = 0 Then
            Throw New InvalidOperationException("There are no activated contexts which could be deactivated.")
        End If
        If Not NativeMethods.DeactivateActCtx(0, m_Cookies.Pop().Value) Then
            Throw New Win32Exception()
        End If
    End Sub

    Public ReadOnly Property Handle() As IntPtr
        Get
            Return m_hActCtx
        End Get
    End Property

    Public Shared Function GetCurrentActivationContext() As ActivationContext
        Dim hActCtx As IntPtr
        If Not NativeMethods.GetCurrentActCtx(hActCtx) Then
            Throw New Win32Exception()
        End If
        For Each ActivationContext As ActivationContext In m_ActivationContexts
            If ActivationContext.Handle = hActCtx Then
                Return ActivationContext
            End If
        Next ActivationContext
        Return Nothing
    End Function

#Region "IDisposable"
    Private m_IsDisposed As Boolean

    Protected Overridable Sub Dispose(ByVal Disposing As Boolean)
        If Not m_IsDisposed Then
            If Disposing Then

                ' TODO: Release unmanaged resources when called explicitly.

                ' Disable enabled activated contexts.
                Do While m_Cookies.Count > 0
                    Dim Cookie As ActivationContextCookie = m_Cookies.Pop()
                    If Not NativeMethods.DeactivateActCtx(0, Cookie.Value) Then
                        Throw New Win32Exception()
                    End If
                Loop

                NativeMethods.ReleaseActCtx(m_hActCtx)
                m_ActivationContexts.Remove(Me)
            End If

            ' TODO: Release unmanaged resources.
        End If
        m_IsDisposed = True
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region
End Class