Option Compare Binary
Option Explicit On
Option Strict On

Imports System.Runtime.InteropServices

' Activation Context API.
Friend NotInheritable Class NativeMethods
    Private Sub New()
        '
    End Sub

#Region "Helpers"
    Friend Shared ReadOnly INVALID_HANDLE_VALUE As IntPtr = New IntPtr(-1)

    Friend Shared Function MAKEINTRESOURCE(ByVal wInteger As Int16) As String
        Return "#" & wInteger
    End Function
#End Region

    <Flags()> _
    Friend Enum ACTCTX_FLAG
        ACTCTX_FLAG_PROCESSOR_ARCHITECTURE_VALID = &H1
        ACTCTX_FLAG_LANGID_VALID = &H2
        ACTCTX_FLAG_ASSEMBLY_DIRECTORY_VALID = &H4
        ACTCTX_FLAG_RESOURCE_NAME_VALID = &H8
        ACTCTX_FLAG_SET_PROCESS_DEFAULT = &H10
        ACTCTX_FLAG_APPLICATION_NAME_VALID = &H20
        ACTCTX_FLAG_SOURCE_IS_ASSEMBLYREF = &H40
        ACTCTX_FLAG_HMODULE_VALID = &H80
    End Enum

    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)> _
    Friend Structure ACTCTX
        Public cbSize As UInt32
        Public dwFlags As ACTCTX_FLAG
        Public lpSource As String
        Public wProcessorArchitecture As PROCESSOR_ARCHITECTURE
        Public wLangId As Int16
        Public lpAssemblyDirectory As String
        Public lpResourceName As String
        Public lpApplicationName As String
        Public hModule As IntPtr
    End Structure

    Friend Enum PROCESSOR_ARCHITECTURE As UInt16
        PROCESSOR_ARCHITECTURE_ALPHA = 2
        PROCESSOR_ARCHITECTURE_ALPHA64 = 7
        PROCESSOR_ARCHITECTURE_ARM = 5
        PROCESSOR_ARCHITECTURE_IA64 = 6
        PROCESSOR_ARCHITECTURE_INTEL = 0
        PROCESSOR_ARCHITECTURE_MIPS = 1
        PROCESSOR_ARCHITECTURE_MSIL = 8
        PROCESSOR_ARCHITECTURE_PPC = 3
        PROCESSOR_ARCHITECTURE_SHX = 4
        PROCESSOR_ARCHITECTURE_UNKNOWN = &HFFFF
    End Enum

    Friend Declare Auto Function CreateActCtx Lib "kernel32.dll" ( _
        ByRef pActCtx As ACTCTX _
    ) As IntPtr

    Friend Declare Sub AddRefActCtx Lib "kernel32.dll" ( _
        ByVal hActCtx As IntPtr _
    )

    Friend Declare Sub ReleaseActCtx Lib "kernel32.dll" ( _
        ByVal hActCtx As IntPtr _
    )

    Friend Declare Function ZombifyActCtx Lib "kernel32.dll" ( _
        ByVal hActCtx As IntPtr _
    ) As Boolean

    Friend Declare Function ActivateActCtx Lib "kernel32.dll" ( _
        ByVal hActCtx As IntPtr, _
        ByRef lpCookie As UInt32 _
    ) As Boolean

    Friend Const DEACTIVATE_ACTCTX_FLAG_FORCE_EARLY_DEACTIVATION As Int32 = &H1

    Friend Declare Function DeactivateActCtx Lib "kernel32.dll" ( _
        ByVal dwFlags As Int32, _
        ByVal ulCookie As UInt32 _
    ) As Boolean

    Friend Declare Function GetCurrentActCtx Lib "kernel32.dll" ( _
        ByRef lphActCtx As IntPtr _
    ) As Boolean

    <StructLayout(LayoutKind.Sequential)> _
    Friend Structure ACTCTX_SECTION_KEYED_DATA_2600
        Public cbSize As UInt32
        Public ulDataFormatVersion As UInt32
        Public lpData As IntPtr
        Public ulLength As UInt32
        Public lpSectionGlobalData As IntPtr
        Public ulSectionGlobalDataLength As UInt32
        Public lpSectionBase As IntPtr
        Public ulSectionTotalLength As UInt32
        Public hActCtx As IntPtr
        Public ulAssemblyRosterIndex As UInt32
    End Structure

    <StructLayout(LayoutKind.Sequential)> _
    Friend Structure ACTCTX_SECTION_KEYED_DATA_ASSEMBLY_METADATA
        Public lpInformation As IntPtr
        Public lpSectionBase As IntPtr
        Public ulSectionLength As UInt32
        Public lpSectionGlobalDataBase As IntPtr
        Public ulSectionGlobalDataLength As UInt32
    End Structure

    <StructLayout(LayoutKind.Sequential)> _
    Friend Structure ACTCTX_SECTION_KEYED_DATA
        Public cbSize As UInt32
        Public ulDataFormatVersion As UInt32
        Public lpData As IntPtr
        Public ulLength As UInt32
        Public lpSectionGlobalData As IntPtr
        Public ulSectionGlobalDataLength As UInt32
        Public lpSectionBase As IntPtr
        Public ulSectionTotalLength As UInt32
        Public hActCtx As IntPtr
        Public ulAssemblyRosterIndex As UInt32
        '// 2600 stops here
        Public ulFlags As UInt32
        Public AssemblyMetadata As ACTCTX_SECTION_KEYED_DATA_ASSEMBLY_METADATA
    End Structure

    Friend Enum FIND_ACTCTX_SECTION_KEY_RETURN_FLAG
        FIND_ACTCTX_SECTION_KEY_RETURN_HACTCTX = &H1
        FIND_ACTCTX_SECTION_KEY_RETURN_FLAGS = &H2
        FIND_ACTCTX_SECTION_KEY_RETURN_ASSEMBLY_METADATA = &H4
    End Enum

    Friend Declare Auto Function FindActCtxSectionString Lib "kernel32.dll" ( _
        ByVal dwFlags As FIND_ACTCTX_SECTION_KEY_RETURN_FLAG, _
        ByRef lpExtensionGuid As IntPtr, _
        ByVal ulSectionId As UInt32, _
        ByRef lpStringToFind As String, _
        ByRef ReturnedData As ACTCTX_SECTION_KEYED_DATA _
    ) As Boolean

    Friend Declare Function FindActCtxSectionGuid Lib "kernel32.dll" ( _
        ByVal dwFlags As FIND_ACTCTX_SECTION_KEY_RETURN_FLAG, _
        ByRef lpExtensionGuid As IntPtr, _
        ByVal ulSectionId As UInt32, _
        ByRef lpStringToFind As IntPtr, _
        ByRef ReturnedData As ACTCTX_SECTION_KEYED_DATA _
    ) As Boolean

    <StructLayout(LayoutKind.Sequential)> _
    Friend Structure ACTIVATION_CONTEXT_BASIC_INFORMATION
        Public hActCtx As IntPtr
        Public dwFlags As Int32
    End Structure

    Friend Const ACTIVATION_CONTEXT_BASIC_INFORMATION_DEFINED As Int32 = &H1

    <Flags()> _
    Friend Enum QUERY_ACTCTX_FLAG
        QUERY_ACTCTX_FLAG_USE_ACTIVE_ACTCTX = &H4
        QUERY_ACTCTX_FLAG_ACTCTX_IS_HMODULE = &H8
        QUERY_ACTCTX_FLAG_ACTCTX_IS_ADDRESS = &H10
        QUERY_ACTCTX_FLAG_NO_ADDREF = &H80000000
    End Enum
End Class