Option Compare Binary
Option Explicit On
Option Strict On

Imports System.Xml

Public Class AssemblyIdentity
    Private m_Attributes As New Dictionary(Of String, IdentityAttribute)
    Private m_IsPolicy As Boolean

    ' Supports both manifest and policy files.
    Public Sub Load(ByVal FileName As String)
        Dim d As New XmlDocument()
        d.Load(FileName)
        Dim nm As New XmlNamespaceManager(d.NameTable)
        nm.AddNamespace("asm.v1", "urn:schemas-microsoft-com:asm.v1")

        ' Attributes are sorted alphabetically.  This is necessary to
        ' compute the correct hash value.
        For Each Attribute As String In New String() {"language", "name", "processorArchitecture", "publicKeyToken", "type", "version"}
            Try
                Dim Value As String = d.DocumentElement.SelectSingleNode("/asm.v1:assembly/asm.v1:assemblyIdentity/@" & Attribute, nm).Value
                Me.Attributes.Add(Attribute, New IdentityAttribute("", Attribute, Value))
                If Attribute = "type" AndAlso Value = "win32-policy" Then
                    m_IsPolicy = True
                End If
            Catch
            End Try
        Next Attribute
    End Sub

    Public Property Attributes() As Dictionary(Of String, IdentityAttribute)
        Get
            Return m_Attributes
        End Get
        Set(ByVal Value As Dictionary(Of String, IdentityAttribute))
            m_Attributes = Value
        End Set
    End Property

    Public Function GetHashValue() As UInt32
        Dim IdentityHash As UInt32 = 0
        For Each Attribute As IdentityAttribute In Me.Attributes.Values

            ' Version number not included in policy hash value.
            If Not m_IsPolicy OrElse Attribute.Name <> "version" Then
                IdentityHash = IdentityHash * CUInt(65599) + Attribute.GetHashValue()
            End If
        Next Attribute
        Return IdentityHash
    End Function

    Public Function GetName() As String
        Dim Language As String
        If Attributes.ContainsKey("language") Then
            Language = Me.Attributes("language").Value
        Else
            Language = "x-ww"
        End If
        Dim Name As String = _
            Me.Attributes("processorArchitecture").Value & "_" & _
            Me.Attributes("name").Value & "_" & _
            Me.Attributes("publicKeyToken").Value & "_"
        If Not m_IsPolicy Then
            Name &= Me.Attributes("version").Value & "_"
        End If
        Name &= _
            Language & "_" & _
            Me.GetHashValue().ToString("x8")
        Return Name
    End Function
End Class