Option Compare Binary
Option Explicit On
Option Strict On

Imports System.IO
Imports System.Text.RegularExpressions
Imports System.Xml

Friend Module Program

    ' No argument validation is performed.
    Public Sub Main(ByVal Args() As String)
        If Args.Length = 0 Then
            Console.WriteLine( _
                "Usage:  updatepkt <certificate file name> <input manifest file name>" & ControlChars.NewLine & _
                "        [ <output manifest file name> ] " & ControlChars.NewLine & _
                "If the last argument is omitted, the input manifest will be overwritten." _
            )
            Return
        End If

        ' Determine the public key token using pktextract.
        Dim psi As New ProcessStartInfo("pktextract", Args(0))
        psi.RedirectStandardOutput = True
        psi.UseShellExecute = False
        Dim p As Process = Nothing
        Try
            p = Process.Start(psi)
        Catch ex As Exception
            Console.WriteLine("Error:  Starting pktextract failed.  ({0})", ex.Message)
        End Try
        p.WaitForExit()
        Dim sr As StreamReader = p.StandardOutput()
        Dim Line As String = sr.ReadLine()
        Do While Line IsNot Nothing
            If InStr(Line, "publicKeyToken=") > 0 Then
                Exit Do
            End If
            Line = sr.ReadLine()
        Loop
        Dim pkt As String = GetPkt(Line)
        Console.WriteLine("Public key token:  """ & pkt & """")

        ' Update the manifest file.
        Dim d As New XmlDocument()
        d.Load(Args(1))
        Dim nm As New XmlNamespaceManager(d.NameTable)
        nm.AddNamespace("asm.v1", "urn:schemas-microsoft-com:asm.v1")
        Dim Node As XmlNode = d.DocumentElement.SelectSingleNode("/asm.v1:assembly/asm.v1:assemblyIdentity/@publicKeyToken", nm)
        Node.Value = pkt
        Dim TargetFileName As String
        If Args.Length > 2 Then
            TargetFileName = Args(2)
        Else
            TargetFileName = Args(1)
        End If
        Try
            d.Save(TargetFileName)
            Console.WriteLine("Output manifest file ""{0}"" saved successfully.", TargetFileName)
        Catch ex As Exception
            Console.WriteLine("Error:  Saving output manifest file failed.  ({0})", ex.Message)
        End Try
    End Sub

    Public Function GetPkt(ByVal Text As String) As String
        Return Regex.Match(Text, "publicKeyToken=""(?<1>[^""]*)""").Groups(1).Value
    End Function
End Module